//
// LogPage.java: Class file for WO Component 'LogPage'
// Project EOInspector
//
// Created by john on Thu Aug 05 2004
//

import java.util.*;
import com.webobjects.foundation.*;
import com.webobjects.appserver.*;
import com.webobjects.eocontrol.*;
import com.webobjects.eoaccess.*;

public class LogPage extends WOComponent {
    protected Logger logger;
    public Log log;
    public String objectID;
    public String selectedTransactionName;
    public String selectedTargetClassName;
    public String selectedMethodName;
    public String selectedUserinfoKey;
    public NSArray logs;
    public int limit;
    public String userInfoKey;

    public LogPage(WOContext context) {
        super(context);
        setLogger(DelegateLogger.defaultLogger());
        limit = 0;
    }

    public Logger logger()
    {
        return logger;
    }

    public void setLogger(Logger newLogger)
    {
        logger = newLogger;
        logs = logger.logs();
    }

    public NSArray transactionNames() {
        NSMutableArray transactionNames = new NSMutableArray();
        Enumeration e = logger.logs().objectEnumerator();
        while (e.hasMoreElements()) {
            String transactionName = ((Log)e.nextElement()).transactionName();
            if ((transactionName != null) &&
                (transactionNames.containsObject(transactionName) == false))
                transactionNames.addObject(transactionName);
        }
        return transactionNames;
    }

    public NSArray targetClassNames() {
        NSMutableArray targetClassNames = new NSMutableArray();
        Enumeration e = logger.logs().objectEnumerator();
        while (e.hasMoreElements()) {
            String className = Inspector.baseClassName(((Log)e.nextElement()).target());
            if (targetClassNames.containsObject(className) == false)
                targetClassNames.addObject(className);
        }
        return targetClassNames;
    }

    public NSArray methodNames() {
        NSMutableArray methodNames = new NSMutableArray();
        Enumeration e = logger.logs().objectEnumerator();
        while (e.hasMoreElements()) {
            String methodName = ((Log)e.nextElement()).methodName();
            if (methodNames.containsObject(methodName) == false)
                methodNames.addObject(methodName);

        }
        return methodNames;
    }

    public NSArray userinfoKeys() {
        NSMutableArray userinfoKeys = new NSMutableArray();
        Enumeration e = logger.logs().objectEnumerator();
        while (e.hasMoreElements()) {
            NSDictionary userinfo = ((Log)e.nextElement()).userinfo();
            if (userinfo != null) {
                Enumeration keyE = userinfo.keyEnumerator();
                while (keyE.hasMoreElements()) {
                    String key = keyE.nextElement().toString();
                    if (userinfoKeys.containsObject(key) == false)
                        userinfoKeys.addObject(key);
                }
            }
        }
        return userinfoKeys;
    }

    public String logTargetName()
    {
        String desc = Inspector.baseClassName(log.target()) + '@' + 
            (String)Integer.toHexString(log.target().hashCode());
        return desc;
    }

    public Date logTimestamp()
    {
        return new Date(log.timestamp());
    }

    public WOComponent filterLogs()
    {
        NSMutableArray quals = new NSMutableArray();
        NSMutableArray removes = new NSMutableArray();
        EOAndQualifier qual;
        EOKeyValueQualifier kvQual;
        
        if (selectedTransactionName != null) {
            kvQual = new EOKeyValueQualifier("transactionName",
                                             EOQualifier.QualifierOperatorEqual,
                                             selectedTransactionName);
            quals.addObject(kvQual);
        }

        if (selectedTargetClassName != null) {
            kvQual = new EOKeyValueQualifier("targetClassName",
                                             EOQualifier.QualifierOperatorEqual,
                                             selectedTargetClassName);
            quals.addObject(kvQual);
        }

        if ((objectID != null) && (objectID.equals("") != false)) {
            kvQual = new EOKeyValueQualifier("objectID",
                                             EOQualifier.QualifierOperatorEqual,
                                             objectID);
            quals.addObject(kvQual);
        }

        if (selectedMethodName != null) {
            kvQual = new EOKeyValueQualifier("methodName",
                                             EOQualifier.QualifierOperatorEqual,
                                             selectedMethodName);
            quals.addObject(kvQual);
        }

        if (selectedUserinfoKey != null) {
            kvQual = new EOKeyValueQualifier("userinfo.allKeys",
                                             EOQualifier.QualifierOperatorContains,
                                             selectedUserinfoKey);
            quals.addObject(kvQual);
        }

        if (quals.count() == 0) {
            logs = logger.logs();            
        } else {
            qual = new EOAndQualifier(quals);
            logs = EOQualifier.filteredArrayWithQualifier(logger.logs(), qual);
        }

        if (limit != 0) {
            logs = new NSArray(logs.objects(new NSRange(0, limit)));
        }

        return null;
    }

    public WOComponent registerUserInfoValueToBrowser()
    {
        Object value = log.userinfo().objectForKey(userInfoKey);
        if (value != null)
            ObjectBrowser.defaultBrowser().registerObject(value);
        return null;
    }

    public WOComponent registerTargetToBrowser()
    {
        ObjectBrowser.defaultBrowser().registerObject(log.target());
        return null;
    }

    public WOComponent removeAllLogs() {
        logger.removeAllLogs();
        return null;
    }

}
