//
//  Logger.java
//  EOInspector
//

import java.util.*;
import com.webobjects.foundation.*;
import com.webobjects.eocontrol.*;

public class Logger {
    NSMutableArray logs;
    Logger parent, root;
    String methodName;
    Object target;
    boolean isTransactionInProgress = false;
    String transactionName;

    public Logger() {
        logs = new NSMutableArray();
    }

    public Logger(Logger parent) {
        logs = new NSMutableArray();
        this.parent = parent;
    }

    public Logger parentLogger() {
        return parent;
    }

    public Logger rootLogger() {
        Logger logger = this;
        while (logger.parentLogger() != null) {
            logger = logger.parentLogger();
        }
        return logger;
    }

    public Object target() {
        return target;
    }

    public void setTarget(Object target) {
        this.target = target;
    }

    public String methodName() {
        return methodName;
    }

    public void setMethodName(String methodName) {
        this.methodName = methodName;
    }

    public NSArray targets() {
        NSMutableArray targets = new NSMutableArray();
        Enumeration e = logs.objectEnumerator();
        while (e.hasMoreElements()) {
            targets.addObject(((Log)e.nextElement()).target());
        }
        return targets;
    }

    public void beginTransaction(String transactionName) {
        isTransactionInProgress = true;
        this.transactionName = transactionName;
    }

    public void endTransaction() {
        isTransactionInProgress = false;
        transactionName = null;
    }

    public void addLog(Object target,
                       String methodName,
                       long timestamp,
                       NSDictionary userinfo) {
        Log log;
        if (isTransactionInProgress == true)
            log = new Log(target, methodName, timestamp, userinfo, transactionName);
        else
            log = new Log(target, methodName, timestamp, userinfo, "WebObjects");
        logs.addObject(log);
    }

    public void addLog(Object target,
                       String methodName,
                       NSDictionary userinfo) {
        addLog(target, methodName, new NSTimestamp().getTime(), userinfo);
    }

    public void addLog(Object target, String methodName) {
        addLog(target, methodName, new NSTimestamp().getTime(), null);
    }

    public NSMutableArray logs() {
        return logs;
    }

    public NSArray logsForMethodName(String methodName, Object target) {
        NSMutableArray filtered = new NSMutableArray();
        Enumeration e = logs.objectEnumerator();
        while (e.hasMoreElements()) {
            Log log = (Log)e.nextElement();
            if (log.methodName().equals(methodName) && log.target().equals(target))
                filtered.addObject(log);
        }
        return filtered;
    }

    public NSArray logsForTarget(Object target) {
        NSMutableArray filtered = new NSMutableArray();
        Enumeration e = logs.objectEnumerator();
        while (e.hasMoreElements()) {
            Log log = (Log)e.nextElement();
            if (log.target() == target)
                filtered.addObject(log);
        }
        return filtered;
    }

    public NSArray removedLogsForMethodName(String methodName, Object target) {
        NSArray filtered = logsForMethodName(methodName, target);
        removeLogs(filtered);
        return filtered;
    }

    public void removeLogs(NSArray logs) {
        this.logs.removeObjectsInArray(logs);
    }

    public void removeAllLogs() {
        logs.removeAllObjects();
    }

    public void update() {
        if (parent != null) {
            removeAllLogs();
            if (methodName == null)
                logs.setArray(parent.logsForTarget(target));
            else
                logs.setArray(parent.logsForMethodName(methodName, target));
        }
    }

    public void update(String methodName, Object target) {
        if (parent != null) {
            removeAllLogs();
            logs.setArray(parent.logsForMethodName(methodName, target));
        }
    }
}
