//
//  ObjectBrowser.java
//  EOInspector
//

import java.util.*;
import com.webobjects.foundation.*;
import com.webobjects.appserver.*;
import com.webobjects.eoaccess.*;
import com.webobjects.eocontrol.*;

public class ObjectBrowser extends WOComponent {
	public static String ObjectRegisteredNotification = "ObjectRegisteredNotification";
	public static String ObjectUnregisteredNotification = "ObjectUnregisteredNotification";
	public static String InspectorKey = "InspectorKey";
	
    private static ObjectBrowser defaultBrowser;
    protected NSMutableArray registeredObjects, registeredInspectors;
    protected NSMutableDictionary nameDictionary, countDictionary;
	protected WorkspacePage workspace;

    public ObjectBrowser(WOContext context) {
        super(context);
        registeredObjects = new NSMutableArray();
        registeredInspectors = new NSMutableArray();
        nameDictionary = new NSMutableDictionary();
        countDictionary = new NSMutableDictionary();
    }

    public static ObjectBrowser defaultBrowser() {
        return defaultBrowser;
    }

    public static void setDefaultBrowser(ObjectBrowser browser) {
        defaultBrowser = browser;
    }

	public WorkspacePage workspace() {
		return workspace;
	}
	
	public NSArray registeredInspectors() {
		return registeredInspectors;
	}
	
    public void registerObject(Object object) {
        String baseName, name;
        Integer countValue;
        int count;
        Inspector inspector;

        if (registeredObjects.containsObject(object) == false) {
            baseName = Inspector.baseClassName(object);
            countValue = (Integer)countDictionary.objectForKey(baseName);
            if (countValue != null) {
                count = countValue.intValue();
                count++;
            } else {
                count = 1;
            }
            name = baseName + " (" + count + ")";

            countDictionary.setObjectForKey(new Integer(count), baseName);
            registeredObjects.addObject(object);
            nameDictionary.setObjectForKey(name, object);
            inspector = InspectPage.inspectorForObject(object);
			inspector.setCount(count);
            registeredInspectors.addObject(inspector);
			NSNotificationCenter.defaultCenter().
				postNotification(ObjectRegisteredNotification, null, new NSDictionary(inspector, InspectorKey));
        }
    }

    public void unregisterObject(Object object) {
		Inspector inspector = null;
        registeredObjects.removeObject(object);
        nameDictionary.removeObjectForKey(object);
		Enumeration e = registeredInspectors.objectEnumerator();
		while (e.hasMoreElements()) {
			Inspector _inspector = (Inspector)e.nextElement();
			
		}
		registeredInspectors.removeObject(inspector);
		NSNotificationCenter.defaultCenter().
			postNotification(ObjectUnregisteredNotification,  null, new NSDictionary(inspector, InspectorKey));

    }

    public void unregisterInspector(Inspector inspector) {
        unregisterObject(inspector.object());
    }        

    public void unregisterAllInspector() {
        registeredInspectors.removeAllObjects();
        registeredObjects.removeAllObjects();
        nameDictionary.removeAllObjects();
        countDictionary.removeAllObjects();
    }

    public String nameForObject(Object object)
    {
        Object name = nameDictionary.objectForKey(object);
        if (name != null)
            return name.toString();
        else
            return null;
    }
	
	//
	// for Groovy
	//
	
	public void add(Object object) {
		registerObject(object);
	}
	
	public ArrayList remove(Object object) {
		unregisterObject(object);;
		return null;
	}
	
}
