//
// WorkspacePage.java: Class file for WO Component 'WorkspacePage'
// Project EOInspector
//
// Created by john on 04/09/22
//

import com.webobjects.foundation.*;
import com.webobjects.appserver.*;
import com.webobjects.eocontrol.*;
import com.webobjects.eoaccess.*;
import groovy.lang.*;
import java.util.*;

public class WorkspacePage extends WOComponent {
    public Transcript transcript;
    protected String script;

    /** @TypeInfo Inspector */
    protected NSMutableArray inspectors;
    public Inspector inspector;
	protected Map bindings;
	protected GroovyShell shell;

    public WorkspacePage(WOContext context) {
        super(context);
		NSNotificationCenter nc = NSNotificationCenter.defaultCenter();
		nc.addObserver(this, new NSSelector("addInspector", new Class[]{NSNotification.class}),
			ObjectBrowser.ObjectRegisteredNotification, null);
		nc.addObserver(this, new NSSelector("removeInspector", new Class[]{NSNotification.class}),
			ObjectBrowser.ObjectUnregisteredNotification, null);
		
		inspectors = new NSMutableArray();
		transcript = new Transcript();
		bindings = new HashMap();
		bindings.put("transcript", transcript);
		bindings.put("browser", ObjectBrowser.defaultBrowser());
		bindings.put("dlogger", DelegateLogger.defaultLogger());
		bindings.put("group", EOModelGroup.defaultGroup());
		bindings.put("sec", EOSharedEditingContext.defaultSharedEditingContext());
		bindings.put("osc", EOObjectStoreCoordinator.defaultCoordinator());
		bindings.put("nc", NSNotificationCenter.defaultCenter());
		bindings.put("app", application());
		bindings.put("session", session());
		bindings.put("ec", session().defaultEditingContext());
		bindings.put("request", context().request());
		bindings.put("response", context().response());
		bindings.put("context", context());
		shell = new GroovyShell(new Binding(bindings));
    }

	public void addInspector(NSNotification n) {
		inspector = (Inspector)n.userInfo().objectForKey(ObjectBrowser.InspectorKey);
		inspectors.addObject(inspector);
		if (bindings.containsValue(inspector.object()) == false)
			bindings.put(inspector.bindingName(), inspector.object());
	}

	public void removeInspector(NSNotification n) {
		inspector = (Inspector)n.userInfo().objectForKey(ObjectBrowser.InspectorKey);
		inspectors.removeObject(inspector);
		bindings.remove(inspector.bindingName());
	}
	
	public String script() {
		return script;
	}
	
	public void setScript(String newScript) {
		script = newScript;
	}
	
	public static String scriptToImportClasses() {
		return "import com.webobjects.foundation.*\n" +
				"import com.webobjects.appserver.*\n" +
				"import com.webobjects.eocontrol.*\n" + 
				"import com.webobjects.eoaccess.*\n";
	}
	
    public WOComponent evaluate()
    {
		try {
			if (script.equals("") == false) {
				Closure cl = (Closure)shell().evaluate(WorkspacePage.scriptToImportClasses() + script);
				cl.call();
			}
		} catch (Exception e) {
			transcript.show(e.getMessage());
		}
        return null;
    }

	public Map bindings() {
		return bindings;
	}
	
	public GroovyShell shell() {
		return shell;
	}
	
    /** @TypeInfo Inspector */
    public NSArray inspectors()
    {
        return inspectors;
    }
	
	public String bindingDescription() {
		return Inspector.baseClassName(inspector.object());
	}
	
	public class Transcript {
		StringBuffer script;
		
		public Transcript() {
			script = new StringBuffer();
		}
		
		public void show(Object object) {
			if (object != null)
				script.append("[" + new Date() + "] " + object.toString() + "\n");
		}
		
		public void clear() {
			script = new StringBuffer();
		}
		
		public String toString() {
			return script.toString();
		}
	}
}
